/*
  main.cpp - Zero Cross Power Control Utility

  Copyright (c) 2021 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <errno.h>

#include "zcpower.h"

ZCPower zcpower;

bool power(bool info)
{
  bool val = false;
  bool rv = zcpower.getPower(val);
  if (!rv)
    printf("[ERROR] ZCPower.getPower()\n");
  else
    printf(info ? "power = %s\n" : "%s\n", val ? "on" : "off");
  return rv;
}

bool pir(bool info)
{
  bool val = false;
  bool rv = zcpower.getPIR(val);
  if (!rv)
    printf("[ERROR] ZCPower.getPIR()\n");
  else
    printf(info ? "pir = %s\n" : "%s\n", val ? "on" : "off");
  return rv;
}

bool level(bool info)
{
  uint8_t val = 0;
  bool rv = zcpower.getLevel(val);
  if (!rv)
    printf("[ERROR] ZCPower.getLevel()\n");
  else
    printf(info ? "level = %u\n" : "%u\n", val);
  return rv;
}

bool limit(bool info)
{
  uint16_t val = 0;
  bool rv = zcpower.getLimit(val);
  if (!rv)
    printf("[ERROR] ZCPower.getLimit()\n");
  else
    printf(info ? "limit = %u\n" : "%u\n", val);
  return rv;
}

bool timer(bool info)
{
  uint16_t val = 0;
  bool rv = zcpower.getTimer(val);
  if (!rv)
    printf("[ERROR] ZCPower.getLimit()\n");
  else
    printf(info ? "timer = %u\n" : "%u\n", val);
  return rv;
}

bool heat(bool info)
{
  int8_t val = 0;
  bool rv = zcpower.getHeat(val);
  if (!rv)
    printf("[ERROR] ZCPower.getHeat()\n");
  else
    printf(info ? "heat = %d\n" : "%d\n", val);
  return rv;
}

bool fan(bool info)
{
  int8_t val = 0;
  bool rv = zcpower.getFan(val);
  if (!rv)
    printf("[ERROR] ZCPower.getFan()\n");
  else
    printf(info ? "fan = %d\n" : "%d\n", val);
  return rv;
}

bool temp(bool info)
{
  int8_t val = 0;
  bool rv = zcpower.getTemp(val);
  if (!rv)
    printf("[ERROR] ZCPower.getTemp()\n");
  else
    printf(info ? "temp = %d\n" : "%d\n", val);
  return rv;
}

bool light(bool info)
{
  uint16_t val = 0;
  bool rv = zcpower.getLight(val);
  if (!rv)
    printf("[ERROR] ZCPower.getLight()\n");
  else
    printf(info ? "light = %u\n" : "%u\n", val);
  return rv;
}

void usage(void)
{
  printf("\n");
  printf("Zero Cross Power Control Utility, Version 1.0\n");
  printf("Copyright (c) 2021 Sasapea's Lab. All right reserved.\n");
  printf("\n");
  printf("Usage: zcpower port-name command\n");
  printf("\n");
  printf("command: power {off|on} ... power off or on\n");
  printf("         pir   {off|on} ... PIR sensor off or on\n");
  printf("         level {n}      ... power level n (0-255)\n");
  printf("         limit {n}      ... power on limit timer n (0-65535) minute\n");
  printf("         timer {n}      ... power on timer n (0-65535) minute\n");
  printf("         heat  {n}      ... heat temperature n (-40-125)\n");
  printf("         fan   {n}      ... fan  temperature n (-40-125)\n");
  printf("         temp           ... SSR  temperature (-40-125)\n");
  printf("         light          ... ambiant light value (0-65535)\n");
  printf("         info           ... display all values\n");
  printf("         reset          ... reset\n");
  printf("         bootloader     ... start bootloader mode\n");
  printf("\n");
  exit(EXIT_FAILURE);
}

int main(int argc, char **argv)
{
  int help;

  if (argc < 3)
    usage();
  if (zcpower.open(argv[1]) < 0)
  {
    printf("[ERROR] ZCPower.open(\"%s\"): %s\n", argv[1], strerror(errno));
    usage();
  }

  for (int i = 2; i < argc; help = false)
  {
    char *p = argv[i++];
    help = true;
    if (strcmp(p, "bootloader") == 0)
    {
      zcpower.reset(0xDFBE); // start bootloader mode
      help = false;
      break;
    }
    else if (strcmp(p, "reset") == 0)
    {
      zcpower.reset();
      help = false;
      break;
    }
    else if (strcmp(p, "power") == 0)
    {
      bool val;
      if (i < argc)
      {
        p = argv[i++];
        if (strcmp(p, "off") == 0)
          val = false;
        else if (strcmp(p, "on") == 0)
          val = true;
        else
          break;
        if (!zcpower.setPower(val))
        {
          printf("[ERROR] ZCPower.setPower(%s)\n", val ? "true" : "false");
          break;
        }
      }
      if (!power(false))
        break;
    }
    else if (strcmp(p, "pir") == 0)
    {
      bool val;
      if (i < argc)
      {
        p = argv[i++];
        if (strcmp(p, "off") == 0)
          val = false;
        else if (strcmp(p, "on") == 0)
          val = true;
        else
          break;
        if (!zcpower.setPIR(val))
        {
          printf("[ERROR] ZCPower.setPIR(%u)\n", val);
          break;
        }
      }
      if (!pir(false))
        break;
    }
    else if (strcmp(p, "level") == 0)
    {
      uint8_t val;
      if (i < argc)
      {
        p = argv[i++];
        val = strtol(p, &p, 10);
        if (*p)
          break;
        if (!zcpower.setLevel(val))
        {
          printf("[ERROR] ZCPower.setLevel(%u)\n", val);
          break;
        }
      }
      if (!level(false))
        break;
    }
    else if (strcmp(p, "limit") == 0)
    {
      uint16_t val;
      if (i < argc)
      {
        p = argv[i++];
        val = strtol(p, &p, 10);
        if (*p)
          break;
        if (!zcpower.setLimit(val))
        {
          printf("[ERROR] ZCPower.setLimit(%u)\n", val);
          break;
        }
      }
      if (!limit(false))
        break;
    }
    else if (strcmp(p, "timer") == 0)
    {
      uint16_t val;
      if (i < argc)
      {
        p = argv[i++];
        val = strtol(p, &p, 10);
        if (*p)
          break;
        if (!zcpower.setTimer(val))
        {
          printf("[ERROR] ZCPower.setLimit(%u)\n", val);
          break;
        }
      }
      if (!timer(false))
        break;
    }
    else if (strcmp(p, "heat") == 0)
    {
      int8_t val;
      if (i < argc)
      {
        p = argv[i++];
        val = strtol(p, &p, 10);
        if (*p)
          break;
        if (!zcpower.setHeat(val))
        {
          printf("[ERROR] ZCPower.setHeat(%d)\n", val);
          break;
        }
      }
      if (!heat(false))
        break;
    }
    else if (strcmp(p, "fan") == 0)
    {
      int8_t val;
      if (i < argc)
      {
        p = argv[i++];
        val = strtol(p, &p, 10);
        if (*p)
          break;
        if (!zcpower.setFan(val))
        {
          printf("[ERROR] ZCPower.setFan(%d)\n", val);
          break;
        }
      }
      if (!fan(false))
        break;
    }
    else if (strcmp(p, "temp") == 0)
    {
      if (!temp(false))
        break;
    }
    else if (strcmp(p, "light") == 0)
    {
      if (!light(false))
        break;
    }
    else if (strcmp(p, "info") == 0)
    {
      if (!power(true))
        break;
      if (!pir(true))
        break;
      if (!level(true))
        break;
      if (!limit(true))
        break;
      if (!timer(true))
        break;
      if (!heat(true))
        break;
      if (!fan(true))
        break;
      if (!temp(true))
        break;
      if (!light(true))
        break;
    }
    else
      break;
  }

  if (zcpower.close() < 0)
    printf("[ERROR] ZCPower.close()\n");

  if (help)
    usage();

  return 0;
}
