/*
 zcpower.h - Zero Cross Power Control Library

 Copyright (c) 2021 Sasapea's Lab. All right reserved.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General
 Public License along with this library; if not, write to the
 Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 Boston, MA  02111-1307  USA
 */
#ifndef _ZCPOWER_H
#define _ZCPOWER_H

#include <stdint.h>
#include <stdbool.h>
#include "serial.h"

//
// Device Command
//
#define ZCPOWER_RESET         0x00
#define ZCPOWER_POWER_ENABLE  0x01 // 0 - 1
#define ZCPOWER_POWER_LEVEL   0x02 // 0 - 255
#define ZCPOWER_POWER_LIMIT   0x03 // 0 - 65535 min
#define ZCPOWER_POWER_TIMER   0x04 // 0 - 65535 min
#define ZCPOWER_PIR_ENABLE    0x05 // 0 - 1
#define ZCPOWER_TEMP_HEAT     0x06 // -40 - 125
#define ZCPOWER_TEMP_FAN      0x07 // -40 - 125
#define ZCPOWER_TEMP_DATA     0x08 // -40 - 125
#define ZCPOWER_LIGHT_DATA    0x09 // 0 - 1023

class ZCPower : public Serial
{
  private:

    uint8_t _count;
    struct
    {
      uint8_t len;
      uint8_t cmd;
      union
      {
        struct { uint16_t val; } reset;
        struct { bool     val; } power;
        struct { uint8_t  val; } level;
        struct { uint16_t val; } limit;
        struct { uint16_t val; } timer;
        struct { bool     val; } pir;
        struct { int8_t   val; } heat;
        struct { int8_t   val; } fan;
        struct { int8_t   val; } temp;
        struct { uint16_t val; } light;
      } arg;
    }__attribute__((packed)) _packet;

    // Hide method
    ssize_t
    read(void *buf, size_t len) override
    {
      return Serial::read (buf, len);
    }

    // Hide method
    ssize_t
    write(const void *buf, size_t len) override
    {
      return Serial::write (buf, len);
    }

    bool
    invoke(uint8_t size = 0)
    {
      uint8_t cmd = _packet.cmd;
      if (write (&_packet, sizeof(_packet.len) + _packet.len)
          == (ssize_t) (sizeof(_packet.len) + _packet.len))
      {
        _count = 0;
        while (read ((uint8_t *) &_packet + _count, 1) == 1)
        {
          if (++_count < sizeof(_packet.len))
            continue;
          if (_count == sizeof(_packet.len) + _packet.len)
            return (_packet.len >= sizeof(_packet.cmd) + size)
                && (_packet.cmd == cmd);
          // invalid packet size ?
          if (_count == sizeof(_packet))
            _count = 0;
        }
      }
      return false;
    }

  public:

    ZCPower(void)
      : _count(0)
    {
    }

    virtual
    ~ZCPower(void) override
    {
    }

    void
    reset(uint16_t mode = 0)
    {
      _packet.len = sizeof(_packet.cmd) + sizeof(_packet.arg.reset);
      _packet.cmd = ZCPOWER_RESET;
      _packet.arg.reset.val = mode;
      invoke();
    }

    bool
    getPower(bool &val)
    {
      _packet.len = sizeof(_packet.cmd);
      _packet.cmd = ZCPOWER_POWER_ENABLE;
      if (!invoke(sizeof(_packet.arg.power)))
        return false;
      val = _packet.arg.power.val;
      return true;
    }

    bool
    setPower(bool val)
    {
      _packet.len = sizeof(_packet.cmd) + sizeof(_packet.arg.power);
      _packet.cmd = ZCPOWER_POWER_ENABLE;
      _packet.arg.power.val = val;
      return invoke();
    }

    bool
    getLevel(uint8_t &val)
    {
      _packet.len = sizeof(_packet.cmd);
      _packet.cmd = ZCPOWER_POWER_LEVEL;
      if (!invoke(sizeof(_packet.arg.level)))
        return false;
      val = _packet.arg.level.val;
      return true;
    }

    bool
    setLevel(uint8_t val)
    {
      _packet.len = sizeof(_packet.cmd) + sizeof(_packet.arg.level);
      _packet.cmd = ZCPOWER_POWER_LEVEL;
      _packet.arg.level.val = val;
      return invoke();
    }

    bool
    getLimit(uint16_t &val)
    {
      _packet.len = sizeof(_packet.cmd);
      _packet.cmd = ZCPOWER_POWER_LIMIT;
      if (!invoke(sizeof(_packet.arg.limit)))
        return false;
      val = _packet.arg.limit.val;
      return true;
    }

    bool
    setLimit(uint16_t val)
    {
      _packet.len = sizeof(_packet.cmd) + sizeof(_packet.arg.limit);
      _packet.cmd = ZCPOWER_POWER_LIMIT;
      _packet.arg.limit.val = val;
      return invoke();
    }

    bool
    getTimer(uint16_t &val)
    {
      _packet.len = sizeof(_packet.cmd);
      _packet.cmd = ZCPOWER_POWER_TIMER;
      if (!invoke(sizeof(_packet.arg.timer)))
        return false;
      val = _packet.arg.timer.val;
      return true;
    }

    bool
    setTimer(uint16_t val)
    {
      _packet.len = sizeof(_packet.cmd) + sizeof(_packet.arg.timer);
      _packet.cmd = ZCPOWER_POWER_TIMER;
      _packet.arg.timer.val = val;
      return invoke();
    }

    bool
    getPIR(bool &val)
    {
      _packet.len = sizeof(_packet.cmd);
      _packet.cmd = ZCPOWER_PIR_ENABLE;
      if (!invoke(sizeof(_packet.arg.pir)))
        return false;
      val = _packet.arg.pir.val;
      return true;
    }

    bool
    setPIR(bool val)
    {
      _packet.len = sizeof(_packet.cmd) + sizeof(_packet.arg.pir);
      _packet.cmd = ZCPOWER_PIR_ENABLE;
      _packet.arg.pir.val = val;
      return invoke();
    }

    bool
    getHeat(int8_t &val)
    {
      _packet.len = sizeof(_packet.cmd);
      _packet.cmd = ZCPOWER_TEMP_HEAT;
      if (!invoke(sizeof(_packet.arg.heat)))
        return false;
      val = _packet.arg.heat.val;
      return true;
    }

    bool
    setHeat(int8_t val)
    {
      _packet.len = sizeof(_packet.cmd) + sizeof(_packet.arg.heat);
      _packet.cmd = ZCPOWER_TEMP_HEAT;
      _packet.arg.heat.val = val;
      return invoke();
    }

    bool
    getFan(int8_t &val)
    {
      _packet.len = sizeof(_packet.cmd);
      _packet.cmd = ZCPOWER_TEMP_FAN;
      if (!invoke(sizeof(_packet.arg.fan)))
        return false;
      val = _packet.arg.fan.val;
      return true;
    }

    bool
    setFan(int8_t val)
    {
      _packet.len = sizeof(_packet.cmd) + sizeof(_packet.arg.fan);
      _packet.cmd = ZCPOWER_TEMP_FAN;
      _packet.arg.fan.val = val;
      return invoke();
    }

    bool
    getTemp(int8_t &val)
    {
      _packet.len = sizeof(_packet.cmd);
      _packet.cmd = ZCPOWER_TEMP_DATA;
      if (!invoke(sizeof(_packet.arg.temp)))
        return false;
      val = _packet.arg.temp.val;
      return true;
    }

    bool
    getLight(uint16_t &val)
    {
      _packet.len = sizeof(_packet.cmd);
      _packet.cmd = ZCPOWER_LIGHT_DATA;
      if (!invoke(sizeof(_packet.arg.light)))
        return false;
      val = _packet.arg.light.val;
      return true;
    }

};

#endif
